/* Aspects. */
.Agda .Comment       { color: #B22222 }
.Agda .Background    {}
.Agda .Markup        { color: #000000 }
.Agda .Keyword       { color: #CD6600 }
.Agda .String        { color: #B22222 }
.Agda .Number        { color: #A020F0 }
.Agda .Symbol        { color: #404040 }
.Agda .PrimitiveType { color: #0000CD }
.Agda .Pragma        { color: black   }
.Agda .Operator      {}
.Agda .Hole          { background: #B4EEB4 }

/* NameKinds. */
.Agda .Bound                  { color: black   }
.Agda .Generalizable          { color: black   }
.Agda .InductiveConstructor   { color: #008B00 }
.Agda .CoinductiveConstructor { color: #8B7500 }
.Agda .Datatype               { color: #0000CD }
.Agda .Field                  { color: #EE1289 }
.Agda .Function               { color: #0000CD }
.Agda .Module                 { color: #A020F0 }
.Agda .Postulate              { color: #0000CD }
.Agda .Primitive              { color: #0000CD }
.Agda .Record                 { color: #0000CD }

/* OtherAspects. */
.Agda .DottedPattern        {}
.Agda .UnsolvedMeta         { color: black; background: yellow         }
.Agda .UnsolvedConstraint   { color: black; background: yellow         }
.Agda .TerminationProblem   { color: black; background: #FFA07A        }
.Agda .IncompletePattern    { color: black; background: #F5DEB3        }
.Agda .Error                { color: red;   text-decoration: underline }
.Agda .TypeChecks           { color: black; background: #ADD8E6        }
.Agda .Deadcode             { color: black; background: #808080        }
.Agda .ShadowingInTelescope { color: black; background: #808080        }

/* Standard attributes. */
.Agda a { text-decoration: none }
.Agda a[href]:hover { background-color: #B4EEB4 }
.Agda [href].hover-highlight { background-color: #B4EEB4; }
